@@ -20,15 +20,14 @@
 
 #include "coursefiltermodel.h"
 #include "models/coursemodel.h"
-
+#include <KLocalizedString>
 #include <QSortFilterProxyModel>
-
-#include <KLocale>
-#include <KDebug>
+#include <QVariant>
+#include "artikulate_debug.h"
 
 CourseFilterModel::CourseFilterModel(QObject* parent)
     : QSortFilterProxyModel(parent)
-    , m_courseModel(0)
+    , m_courseModel(nullptr)
     , m_view(CourseFilterModel::AllResources)
 {
 
@@ -47,6 +46,7 @@ void CourseFilterModel::setView(CourseFilterModel::CourseResourceView view)
     m_view = view;
     invalidateFilter();
     emit viewChanged();
+    emit filteredCountChanged();
 }
 
 CourseFilterModel::CourseResourceView CourseFilterModel::view() const
@@ -54,16 +54,27 @@ CourseFilterModel::CourseResourceView CourseFilterModel::view() const
     return m_view;
 }
 
-void CourseFilterModel::setCourseModel(CourseModel* courseModel)
+void CourseFilterModel::setCourseModel(CourseModel *courseModel)
 {
     if (courseModel == m_courseModel) {
         return;
     }
-
+    if (m_courseModel) {
+        disconnect(m_courseModel, &CourseModel::languageChanged,
+                this, &CourseFilterModel::filteredCountChanged);
+        disconnect(m_courseModel, &CourseModel::rowCountChanged,
+                this, &CourseFilterModel::filteredCountChanged);
+    }
     m_courseModel = courseModel;
+    connect(m_courseModel, &CourseModel::languageChanged,
+            this, &CourseFilterModel::filteredCountChanged);
+    connect(m_courseModel, &CourseModel::rowCountChanged,
+            this, &CourseFilterModel::filteredCountChanged);
+
     setSourceModel(m_courseModel);
     sort(0);
     emit courseModelChanged();
+    emit filteredCountChanged();
 }
 
 int CourseFilterModel::filteredCount() const
@@ -71,7 +82,7 @@ int CourseFilterModel::filteredCount() const
     return rowCount();
 }
 
-bool CourseFilterModel::lessThan(const QModelIndex& left, const QModelIndex& right) const
+bool CourseFilterModel::lessThan(const QModelIndex &left, const QModelIndex &right) const
 {
     return QSortFilterProxyModel::lessThan(left, right);
 }
@@ -91,3 +102,8 @@ bool CourseFilterModel::filterAcceptsRow(int sourceRow, const QModelIndex &sourc
         return true;
     }
 }
+
+QVariant CourseFilterModel::course(int row) const
+{
+    return m_courseModel->data(m_courseModel->index(row, 0), CourseModel::DataRole);
+}
